/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;

import de.admadic.papers.reflection.log.Logger;

/**
 * Provides an abstract inspector which implements default behaviour for 
 * an actual inspector.
 * 
 * @author Rainer Schwarze
 * @param <T> The type which the inspector takes care of (for instance Class or Method).
 */
public abstract class AbstractInspector<T> implements IInspector<T>  {
	static Logger logger = Logger.getLogger(AbstractInspector.class);

	final private T item;	// the actual item which the actual inspector analyses

	// house keeping:
	private TYPE type = TYPE.UNKNOWN;
	private boolean expanded = false;
	private boolean expandable = false;

	private String label = "";
	private String labelSuffix = "";
	private String shortInfo;
	private String detailedInfo;

	private HashMap<String, String> detailsList = new LinkedHashMap<String, String>();
	long detailsListMark = 0L;
	long detailsQueriedMark = -1;
	StringBuffer detailsBuffer = new StringBuffer();

//	/**
//	 * Creates an instance with the given label and item. The type is assumed
//	 * to be <code>UNKNOWN</code>. (This constructor actually calls 
//	 * AbstractInspector(String, T, TYPE.UNKNOWN) .)
//	 * 
//	 * @param label The label of this inspector.
//	 * @param item	The item which this inspector analyses.
//	 */
//	public AbstractInspector(String label, T item) {
//		this(label, item, TYPE.UNKNOWN);
//	}

	/**
	 * Creates an instance with the given label, item and type.
	 * 
	 * @param label The label of this inspector.
	 * @param item	The item which this inspector analyses.
	 * @param type 	The type of this inspector.
	 */
	public AbstractInspector(String label, T item, TYPE type) {
		super();
		this.label = label;
		this.item = item;
		this.type = type;
		logger.debug("abstract instantiated");
	}
	
	/**
	 * Returns the item of the inspector.
	 * 
	 * @return the item
	 */
	protected T getItem() {
		return item;
	}

	/**
	 * Sets the basic detailed information. See {@link #addDetail(String, String)}
	 * for adding a key/value pair.
	 * 
	 * @param detailedInfo the detailedInfo to set
	 */
	protected synchronized void setDetailedInfo(String detailedInfo) {
		this.detailedInfo = detailedInfo;
		detailsListMark++;
	}

	/**
	 * Sets the short information.
	 * 
	 * @param shortInfo the shortInfo to set
	 */
	protected synchronized void setShortInfo(String shortInfo) {
		this.shortInfo = shortInfo;
	}

	/**
	 * Returns the detailed information of this inspector.
	 * The detailed description includes the basic information and the
	 * key/value pairs.
	 * 
	 * @return	Returns a detailed description of the item.
	 * @see de.admadic.papers.reflection.inspectors.IInspector#getDetailedInfo()
	 */
	public synchronized String getDetailedInfo() {
		if (detailsListMark!=detailsQueriedMark) {
			// rebuild
			detailsBuffer.delete(0, detailsBuffer.length());
			if (detailedInfo!=null) {
				detailsBuffer.append(detailedInfo);
				detailsBuffer.append("\n");
			}
			for (String k : detailsList.keySet()) {
				detailsBuffer.append(k);
				detailsBuffer.append(" : ");
				detailsBuffer.append(detailsList.get(k));
				detailsBuffer.append("\n");
			}
			detailsQueriedMark = detailsListMark;
		}
		return detailsBuffer.toString();
	}

	/**
	 * Adds a key/value pair to the detail information.
	 * 
	 * @param name
	 * @param value
	 */
	public synchronized void addDetail(String name, String value) {
		detailsList.put(name, value);
		detailsListMark++;
	}
	
	/**
	 * Returns the short information of this inspector.
	 * 
	 * @return	Returns a short description of the item.
	 * @see de.admadic.papers.reflection.inspectors.IInspector#getShortInfo()
	 */
	public synchronized String getShortInfo() {
		return shortInfo;
	}

	/**
	 * Returns true, if this inspector is already expanded.
	 * 
	 * @return the expanded
	 */
	public synchronized boolean isExpanded() {
		return expanded;
	}

	/**
	 * Sets the expanded state of this inspector.
	 * Usually this function is called by the expansion helper. You should
	 * not call this function directly unless you know what you are doing.
	 * 
	 * @param expanded the expanded to set
	 */
	public synchronized void setExpanded(boolean expanded) {
		this.expanded = expanded;
	}

	/**
	 * Returns true, if this inspector is expandable.
	 * Subclasses can either set the expandable flag using 
	 * {@link #setExpandable(boolean)} or by overriding this method.
	 * The default value for the expandable flag is <code>false</code>.
	 * 
	 * @return the expandable
	 */
	public boolean isExpandable() {
		return expandable;
	}

	/**
	 * Sets the expandable flag of this inspector.
	 * Subclasses can either set the expandable flag using 
	 * this method or by overriding {@link #isExpandable()}.
	 * The default value for the expandable flag is <code>false</code>.
	 * 
	 * @param expandable the expandable to set
	 */
	protected void setExpandable(boolean expandable) {
		this.expandable = expandable;
	}

	abstract public void expand(ArrayList<IInspector<?>> list);

	/**
	 * Returns the label of this inspector.
	 * Currently there is a label and a label suffix. This method returns
	 * the label with the appended label suffix. The suffix is used in array 
	 * expansion to identify the elements (for instance "[4]").
	 * Note that the support for suffix may be removed and array labels are 
	 * directly put into the label field.
	 * 
	 * @return the label
	 */
	public String getLabel() {
		return label + labelSuffix;	// FIXME: perf
	}

	/**
	 * Sets the label of this inspector.
	 * 
	 * @param label the label to set
	 */
	public synchronized void setLabel(String label) {
		this.label = label;
	}

	/**
	 * @return	Returns a String information about this instance.
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		// return "<html>" + getLabel() + " : <b>" + getShortInfo() + "</b> (" + getItemTypeInfo() + ")";
		return " " + getLabel() + " : " + getShortInfo() + " ";
	}

	/**
	 * Sets the label suffix. 
	 * Note that support for label suffices may be removed in the future.
	 * See {@link #getLabel()}.
	 * 
	 * @param labelSuffix the labelSuffix to set
	 */
	public synchronized void setLabelSuffix(String labelSuffix) {
		this.labelSuffix = labelSuffix;
	}

	/**
	 * Returns the type of this inspector.
	 * 
	 * @return the type
	 */
	public synchronized TYPE getType() {
		return type;
	}

	/**
	 * Sets the type of this inspector.
	 * 
	 * @param type the type to set
	 */
	public synchronized void setType(TYPE type) {
		this.type = type;
	}
}
