/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.util.ArrayList;
import java.util.List;

import de.admadic.papers.reflection.log.Logger;

/**
 * Provides an Inspector for an array of elements.
 * (This should be used for cases like the list of declared fields of
 * a class.)
 * 
 * @author Rainer Schwarze
 * @param <T>	The array element type. 
 */
public class ArrayInspector<T> extends AbstractInspector<List<T>> {
	@SuppressWarnings("hiding")
	static Logger logger = Logger.getLogger(ArrayInspector.class);

	private Class<?> forcedItemType = null;

	/**
	 * Creates an instance with the given parameters and sets the type
	 * field to TYPE.UNKNOWN.
	 * 
	 * @param label	The label for this array of inspectors.
	 * @param item	A List<T> of items.
	 */
	public ArrayInspector(String label, List<T> item) {
		this(label, item, TYPE.UNKNOWN);
	}

	/**
	 * Creates an instance with the given parameters.
	 * 
	 * @param label The label for this array of inspectors.
	 * @param item	A List<T> of items.
	 * @param type 	The type of this array of inspectors.
	 */
	public ArrayInspector(String label, List<T> item, TYPE type) {
		this(label, item, type, null);
	}

	/**
	 * Creates an instance with the given parameters.
	 * 
	 * @param label The label for this array of inspectors.
	 * @param item	A List<T> of items.
	 * @param type 	The type of this array of inspectors.
	 * @param forcedItemType 
	 */
	public ArrayInspector(String label, List<T> item, TYPE type, Class<?> forcedItemType) {
		super(label, item, type);
		this.forcedItemType = forcedItemType;

		if (item.size()>0) {
			T t = item.get(0);
			setShortInfo(t.getClass().getSimpleName() + "[" + item.size() + "]");
			addDetail("array length", ""+item.size());
			addDetail("element type", t.getClass().getSimpleName());
			addDetail("element type (canonical)", t.getClass().getCanonicalName());
		} else {
			setShortInfo("unknown[0]");

			addDetail("array length", ""+item.size());
			addDetail("element type", "unknown");
		}
	}

	/**
	 * Expands this inspector and fills the given list with the inspectors for 
	 * each item in the item list of this inspector.
	 * 
	 * @param list
	 * @see de.admadic.papers.reflection.inspectors.AbstractInspector#expand(ArrayList)
	 */
	@Override
	public synchronized void expand(ArrayList<IInspector<?>> list) {
		List<T> itemList = getItem();
		for (int i=0; i<itemList.size(); i++) {
			logger.debug("expanding item #" + i);
			T t = itemList.get(i);
			AbstractInspector<?> s = Factory.create("[" + i + "]", t, forcedItemType);
			list.add(s);
		}
	}

	/**
	 * This inspector is expandable if it has at least one element in the
	 * item list.
	 * 
	 * @return	Returns true, if the array contains at least one element.
	 * @see de.admadic.papers.reflection.inspectors.IInspector#isExpandable()
	 */
	@Override
	public boolean isExpandable() {
		return getItem().size()!=0;
	}
}
