/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;

/**
 * Provides an inspector for a class.
 * 
 * @author Rainer Schwarze
 */
public class ClassInspector extends AbstractInspector<Class> {

	/**
	 * Creates an instance with the given parameters.
	 * 
	 * @param label The label of this inspector.
	 * @param item	The Class to be inspected.
	 */
	public ClassInspector(String label, Class item) {
		super(label, item, TYPE.CLASS);
		if (item!=null) {
			setShortInfo(item.getSimpleName());
			addDetail("type", item.getCanonicalName());
			setExpandable(true);
		} else {
			setShortInfo("(null)");
			setDetailedInfo("<nothing>");
			setExpandable(false);
		}
	}

	/**
	 * @param list
	 * @see de.admadic.papers.reflection.inspectors.AbstractInspector#expand(ArrayList)
	 */
	@Override
	public synchronized void expand(ArrayList<IInspector<?>> list) {
		Class cls = getItem();

		ArrayList<IInspector<?>> gilist = new ArrayList<IInspector<?>>(); 
		GroupInspector gi = new GroupInspector("misc. information", gilist, TYPE.UNKNOWN, "for " + getItem().getSimpleName());
		
		// simple property stuff:
		list.add(new SimplePropertyInspector("name", cls.getName()));

		list.add(gi);

		gilist.add(new SimplePropertyInspector("simple name", cls.getSimpleName()));
		gilist.add(new SimplePropertyInspector("canonical name", cls.getCanonicalName()));
		gilist.add(new SimplePropertyInspector("modifiers", Util.modifiers2String(cls.getModifiers())));

		addDetail("name", cls.getName());
		addDetail("simple name", cls.getSimpleName());
		addDetail("canonical name", cls.getCanonicalName());
		addDetail("modifiers", Util.modifiers2String(cls.getModifiers()));

		// the simple elements of a class:
		list.add(new ArrayInspector<Field>(
				"declared fields", 
				Util.makeList(cls.getDeclaredFields()), 
				TYPE.FIELD));
		list.add(new ArrayInspector<Method>(
				"declared methods", 
				Util.makeList(cls.getDeclaredMethods()), 
				TYPE.METHOD));
		list.add(new ArrayInspector<Constructor>(
				"declared constructors", 
				Util.makeList(cls.getDeclaredConstructors()), 
				TYPE.CONSTRUCTOR));
		list.add(new ArrayInspector<Class>(
				"declared classes",
				Util.makeList(cls.getDeclaredClasses()),
				TYPE.CLASS));

		// implemented interfaces:
		list.add(new ArrayInspector<Class>(
				"interfaces",
				Util.makeList(cls.getInterfaces()),
				TYPE.CLASS));

		// if its an enum, handle the constants:
		gilist.add(new SimplePropertyInspector("isEnum", Util.boolean2String(cls.isEnum())));
		if (cls.isEnum()) {
			list.add(new ArrayInspector<Object>(
					"enum constants",
					Util.makeList(cls.getEnumConstants()),
					TYPE.CLASS));
		}

		// a couple of other flags (isXxx methods):
		gilist.add(new SimplePropertyInspector("isAnonymousClass", Util.boolean2String(cls.isAnonymousClass())));
		gilist.add(new SimplePropertyInspector("isArray", Util.boolean2String(cls.isArray())));
		if (cls.isArray()) {
			list.add(new ClassInspector("component type", cls.getComponentType()));
		}
		gilist.add(new SimplePropertyInspector("isInterface", Util.boolean2String(cls.isInterface())));
		gilist.add(new SimplePropertyInspector("isLocalClass", Util.boolean2String(cls.isLocalClass())));
		gilist.add(new SimplePropertyInspector("isMemberClass", Util.boolean2String(cls.isMemberClass())));
		if (cls.isMemberClass()) {
			list.add(new ClassInspector("declaring class", cls.getDeclaringClass()));
		}
		gilist.add(new SimplePropertyInspector("isPrimitive", Util.boolean2String(cls.isPrimitive())));
		gilist.add(new SimplePropertyInspector("isSynthetic", Util.boolean2String(cls.isSynthetic())));

		// sub/super classes, enclosing classes etc.:
		list.add(new ClassInspector("super class", cls.getSuperclass()));
		list.add(new ClassInspector("enclosing class", cls.getEnclosingClass()));
		list.add(new ConstructorInspector("enclosing constructor", cls.getEnclosingConstructor()));
		list.add(new MethodInspector("enclosing method", cls.getEnclosingMethod()));

		// the generics stuff:
		TypeInspector.expand(cls, list); // atually does nothing - its here as a marker	
		Helper.expandGenericDeclaration(cls, list);
		list.add(
				Factory.create(
						"generic super class", 
						cls.getGenericSuperclass()));
		list.add(new ArrayInspector<Type>(
				"generic interfaces",
				Util.makeList(cls.getGenericInterfaces()),
				TYPE.TYPE));

		// this is for getDeclaredAnnotations:
		Helper.expandAnnotatedElement(cls, list);

		list.add(new PackageInspector("package", cls.getPackage()));
	}
}
