/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Proxy;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.WildcardType;
import java.util.HashMap;

import de.admadic.papers.reflection.log.Logger;

/**
 * Provides a factory for creating inspectors based on the item for which
 * they are responsible.
 * An easy case is a <code>ClassInspector</code>, it will be responsible for 
 * a <code>Class</code>. Things get a bit more complicated when items 
 * only implement a specific interface for which an inspector is known.
 * So the create method actually does this:
 * 
 * <ol>
 * <li>Is item.getClass() in the hash map of item:inspectors?</li>
 * <li>if yes, retrieve the inspector and create an instance, otherwise continue</li>
 * <li>Get a list of implemented interfaces and check whether any of them 
 * 		exist in the item:inspector hash map.</li>
 * <li>if an entry was found, retrieve the inspector and create an instance, 
 * 		otherwise continue</li>
 * <li>retrieve the super class and if there is any go to step #1.</li>
 * <li>If nothing was found, signal an error.</li>
 * </ol>
 * 
 * @author Rainer Schwarze
 */
public class Factory {
	static Logger logger = Logger.getLogger(Factory.class);
	// FIXME: do we need generic type parameters in these methods any more? looks like we could go without them... check it.

	// use a hasp map to link between items to be inspected and their
	// corresponding Inspectors:
	static private HashMap<Class<?>, Class<? extends AbstractInspector<?>>> inspectors = 
		new HashMap<Class<?>, Class<? extends AbstractInspector<?>>>();
	static {
		inspectors.put(Method.class, MethodInspector.class);
		inspectors.put(Constructor.class, ConstructorInspector.class);
		inspectors.put(Field.class, FieldInspector.class);
		inspectors.put(GenericArrayType.class, GenericArrayTypeInspector.class);
		inspectors.put(ParameterizedType.class, ParameterizedTypeInspector.class);
		inspectors.put(WildcardType.class, WildCardTypeInspector.class);
		inspectors.put(Class.class, ClassInspector.class);
		inspectors.put(TypeVariable.class, TypeVariableInspector.class);
		inspectors.put(Proxy.class, ProxyInspector.class);
		inspectors.put(Annotation.class, AnnotationInspector.class);
	}

	/**
	 * Helper method - find the class which is the class of the item, one
	 * of its implemented interfaces, a super class or one of the interfaces
	 * implemented in any super class which is registered in the inspectors 
	 * hash map.
	 * Note that this method does not return the inspector class, but the 
	 * item class. That item class can be used to retrieve an 
	 * inspector class from the inspectors hash map.
	 * 
	 * @param <T>
	 * @param t	The item to get the registered item class for.
	 * @return	Returns the class which matches an item class in the hash map 
	 * 			or <code>null</code>, if no entry was found.
	 */
	static private <T> Class<?> lookupClass(T t) {
		Class<? extends AbstractInspector<?>> cls = null;
		// lets try a simple first shot:
		cls = inspectors.get(t.getClass());
		if (cls!=null) return cls;

		// simple shot failed, lets go through interfaces and superclasses:
		logger.warn("class " + t.getClass().getCanonicalName() + " not in lookup list");
		Class<?> clasz = t.getClass();
		while (clasz!=null) {
			logger.warn("checking class " + clasz.getCanonicalName());
			if (inspectors.containsKey(clasz)) {
				logger.warn("super-class " + clasz.getCanonicalName() + " is in lookup list");
				return clasz;
			}
			Class<?> [] itfs = t.getClass().getInterfaces();
			for (Class<?> itf : itfs) {
				logger.warn("checking interface " + itf.getCanonicalName());
				if (inspectors.containsKey(itf)) {
					logger.warn("interface " + itf.getCanonicalName() + " is in lookup list");
					return itf;
				}
			}
			clasz = clasz.getSuperclass();
		}
		logger.warn("nothing found in lookup list for " + t.getClass().getCanonicalName());

		return null;
	}

	/**
	 * Creates an Inspector instance for the given item.
	 * This method will always return an Inspector. In case
	 * of errors an ErrorInspector is returned.
	 * 
	 * @param <T>
	 * @param label
	 * @param t
	 * @return	Returns the inspector created for the specified item t.
	 */
	static public <T> AbstractInspector<?> create(String label, T t) {
		return create(label, t, null);
	}

	/**
	 * Creates an Inspector instance for the given item.
	 * This method will always return an Inspector. In case
	 * of errors an ErrorInspector is returned.
	 * 
	 * @param <T>
	 * @param label
	 * @param t
	 * @param forcedItemType 
	 * @return	Returns the inspector created for the specified item t.
	 */
	static public <T> AbstractInspector<?> create(String label, T t, Class<?> forcedItemType) {
		if (t==null) {
			return new SimplePropertyInspector(label, "(null)");
		}
		Class<? extends AbstractInspector<?>> cls = null;
		Class<?> matchingcls = t.getClass();
		if (forcedItemType!=null) {
			matchingcls = forcedItemType;
		}
		cls = inspectors.get(matchingcls);
		if (cls==null) {
			if (forcedItemType!=null) {
				// throw
				return new ErrorInspector("Error", 
						new IllegalArgumentException("item type was forced, but not registered as key for inspector " + forcedItemType.getCanonicalName())
					);
			}
			matchingcls = lookupClass(t);
			if (matchingcls!=null) {
				cls = inspectors.get(matchingcls);
			}
		}
		if (cls==null) {
			// throw
			return new ErrorInspector("Error", 
					new IllegalArgumentException("could not find an inspector for class " + t.getClass().getName())
				);
		}
		try {
			Constructor<? extends AbstractInspector<?>> ctor = cls.getConstructor(
					new Class[]{String.class, matchingcls});
			AbstractInspector<?> ai = ctor.newInstance(new Object[]{label, t});
			return ai;
		} catch (Exception e) {
			return new ErrorInspector("Error", e);
//			e.printStackTrace();
		}
	}
}
