/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.lang.annotation.Annotation;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.GenericDeclaration;
import java.lang.reflect.Member;
import java.lang.reflect.TypeVariable;
import java.util.ArrayList;

/**
 * Provides a few helper methods for inspector expansion.
 * The helper methods usually apply to interfaces declared 
 * in the reflection API. 
 * 
 * For instance the Member interface is implemented by
 * Field, Constructor and Method. The information which is available via the 
 * Member interface can be "expanded" in the FieldInspector, ConstructorInspector
 * and MethodInspector. To share the code for such kind of expansion, there
 * is an <code>expandMember(...)</code> method which shall be used in the 
 * inspectors for Field, Method and Constructor.
 * 
 * @author Rainer Schwarze
 */
public class Helper {

	/**
	 * Expands the information which is available through the java.lang.reflect.GenericDeclaration
	 * interface.
	 * 
	 * @param <T>	The actual type implementing the GenericDeclaration interface.
	 * @param item	The item of the type T (for instance Class, etc).
	 * @param list	The list of inspectors which is filled by this method.
	 */
	static public <T extends GenericDeclaration> void expandGenericDeclaration(T item, ArrayList<IInspector<?>> list) {
		list.add(new ArrayInspector<TypeVariable<?>>(
				"type parameters", Util.makeList(item.getTypeParameters()), TYPE.TYPE));
	}

	/**
	 * Expands the information which is available through the java.lang.reflect.AnnotatedElement
	 * interface.
	 * 
	 * @param <T>	The actual type implementing the AnnotatedElement interface.
	 * @param item	The item of the type T (for instance Class, Method, etc).
	 * @param list	The list of inspectors which is filled by this method.
	 */
	static public <T extends AnnotatedElement> void expandAnnotatedElement(T item, ArrayList<IInspector<?>> list) {
		list.add(
				new ArrayInspector<Annotation>(
						"annotations", 
						Util.makeList(item.getAnnotations()), 
						TYPE.ANNOTATION, 
						Annotation.class));
		list.add(
				new ArrayInspector<Annotation>(
						"declared annotations", 
						Util.makeList(item.getDeclaredAnnotations()), 
						TYPE.ANNOTATION, 
						Annotation.class));
	}

	/**
	 * Expands the information which is available through the java.lang.reflect.AnnotatedElement
	 * interface.
	 * 
	 * @param <T>	
	 * @param item	The item of the type T (for instance Class, Method, etc).
	 * @param list	The list of inspectors which is filled by this method.
	 * @param parameterTypes 
	 * @param parameterAnnotations 
	 */
	static public <T> void expandParameterAnnotations(
			T item, ArrayList<IInspector<?>> list,
			Class<?>[] parameterTypes, Annotation[][] parameterAnnotations) {
		ArrayList<IInspector<?>> gilist = new ArrayList<IInspector<?>>();
		GroupInspector gi = new GroupInspector(
				"parameter annotations",
				gilist,
				TYPE.ANNOTATION,
				"Annotation[" + parameterAnnotations.length + "][?]");
		list.add(gi);
		if (parameterTypes.length!=parameterAnnotations.length) {
			gilist.add(new ErrorInspector(
					"Error",
					new IllegalArgumentException(
							"paramTypes[].length==" + parameterTypes.length +
							" but paramAntns[][].length==" + parameterAnnotations.length +
							" - this is not yet supported.")));
			return;
		}
		int pcount = parameterAnnotations.length;
		for (int pi=0; pi<pcount; pi++) {
			gilist.add(
					new ArrayInspector<Annotation>(
							"["+pi+"] atns for " + parameterTypes[pi].getName(),
							Util.makeList(parameterAnnotations[pi]),
							TYPE.ANNOTATION,
							Annotation.class));
		}
	}

	/**
	 * Expands the information which is available through the java.lang.reflect.Member
	 * interface.
	 * 
	 * @param <T>	The actual type implementing the Member interface.
	 * @param item	The item of the type T (for instance Field, Method, Constructor).
	 * @param list	The list of inspectors which is filled by this method.
	 */
	static public <T extends Member> void expandMember(T item, ArrayList<IInspector<?>> list) {
		list.add(new SimplePropertyInspector("name", item.getName()));
		list.add(new SimplePropertyInspector("modifiers", Util.modifiers2String(item.getModifiers())));
		list.add(new SimplePropertyInspector("isSynthetic", Util.boolean2String(item.isSynthetic())));
	}
}
