/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.util.ArrayList;


/**
 * Provides the interface for all inspectors.
 * Note that inspectors do not own child inspectors. They are used within
 * node instances which maintain their children (which are also nodes). 
 * The user objects of the child nodes are the "child inspectors" of the 
 * "parent inspector".
 * 
 * @author Rainer Schwarze
 *
 * @param <T>	The type which the inspector takes care of (for instance Class or Method).
 */
public interface IInspector<T> {

	/**
	 * Returns the label of the inspector. This is the information which is 
	 * usually displayed in the nodes of the tree view. (For instance 
	 * "declared fields" for the list of declared fields and "[3]" for 
	 * the fourth entry in an array.)
	 * 
	 * @return	Returns the label for this inspector.
	 */
	abstract public String getLabel();

	/**
	 * Returns a short information which is usually shown in the nodes of the
	 * tree view after the label. (For instance in the list of methods the
	 * 8th method with the name <code>isSynthetic</code> would be shown as 
	 * "[7] : isSynthetic".)
	 * 
	 * @return	Returns a short description about this item.
	 */
	abstract public String getShortInfo();

	/**
	 * Returns a detailed description of the inspector which is shown in the 
	 * details field when the node of this inspector has been selected.
	 * 
	 * @return	Returns a detailed description about this item.
	 */
	abstract public String getDetailedInfo();

	/**
	 * Returns true, if the inspector is expandable i.o.W. if it can create
	 * sub inspectors. For instance a class inspector can expand to include at
	 * least a list of declared fields, etc. However a simple property inspector
	 * (one which shows the name of an item) is not expandable. 
	 * 
	 * @return	Returns true if the item is expandable.
	 */
	abstract public boolean isExpandable();

	/**
	 * Returns true, if this inspector has already been expanded.
	 *  
	 * Non-expandable inspectors should return false here to be sure. 
	 * (But it doesn't matter now if they return true.)
	 * 
	 * @return	Returns true, if the inspector is already expanded.
	 */
	public abstract boolean isExpanded();

	/**
	 * Sets the type of the inspector. See {@link TYPE} for more information.
	 * 
	 * @param type	The TYPE to set for this inspector.
	 */
	public abstract void setType(TYPE type);

	/**
	 * Returns the type of this inspector. See {@link TYPE} for more information.
	 * 
	 * @return	Returns the type of this inspector.
	 */
	public abstract TYPE getType();


	/**
	 * This method is called when the inspector is expanded. 
	 * If the inspector is expandable and is not yet expanded, it should
	 * fill the list of inspectors. It is guaranteed, that this method is
	 * only called, if the inspector is expandable and is not yet expanded.
	 * 
	 * Inspectors which are not expandable should do nothing in this method.
	 * 
	 * FIXME: check whether this method should be part of this interface or another.
	 * 
	 * @param list	The list which shall get filled during expansion.
	 */
	public abstract void expand(ArrayList<IInspector<?>> list);

	/**
	 * The code which calls the <code>expand</code> method uses this method
	 * to mark the inspector as expanded afterwards.
	 * 
	 * FIXME: check whether this method should be part of this interface or another.
	 * 
	 * @param expanded
	 */
	public abstract void setExpanded(boolean expanded);
}
