/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;

/**
 * Provides an inspector for Methods.
 * 
 * @author Rainer Schwarze
 */
public class MethodInspector extends AbstractInspector<Method> {

	/**
	 * Creates an instance with the given parameters.
	 * The parameter <code>item</code> may be <code>null</code>.
	 * 
	 * @param label The label of this inspector.
	 * @param item	The Method to inspect.
	 */
	public MethodInspector(String label, Method item) {
		super(label, item, TYPE.METHOD);
		if (item!=null) {
			setShortInfo(
					Util.makeFuncString(
							item.getName(), 
							item.getParameterTypes()));
			addDetail("generic string", item.toGenericString());
			addDetail("modifiers", Util.modifiers2String(item.getModifiers()));
			addDetail("return type", item.getReturnType().getCanonicalName());
			Class<?> [] parms = item.getParameterTypes();
			int index = 0;
			for (Class<?> p : parms) {
				addDetail("param type #" + index++, p.getCanonicalName());
			}
			setExpandable(true);
		} else {
			setShortInfo("(null)");
			setDetailedInfo("nothing");
			setExpandable(false);
		}
	}

	@Override
	public void expand(ArrayList<IInspector<?>> list) {
		Method item = getItem();
		Helper.expandMember(item, list);
		Helper.expandAnnotatedElement(item, list);
		Helper.expandGenericDeclaration(item, list);

		list.add(new SimplePropertyInspector("isVarArgs", Util.boolean2String(item.isVarArgs())));
		list.add(new SimplePropertyInspector("isBridge", Util.boolean2String(item.isBridge())));
		
		list.add(new ArrayInspector<Class<?>>(
				"exception types", Util.makeList(item.getExceptionTypes()), TYPE.CLASS));
		list.add(new ArrayInspector<Type>(
				"generic exception types", Util.makeList(item.getGenericExceptionTypes()), TYPE.TYPE));
		list.add(new ArrayInspector<Class<?>>(
				"parameter types", Util.makeList(item.getParameterTypes()), TYPE.CLASS));
		list.add(new ArrayInspector<Type>(
				"generic parameter types", Util.makeList(item.getGenericParameterTypes()), TYPE.TYPE));
		list.add(new ClassInspector("return type", item.getReturnType()));
		list.add(Factory.create("generic return type", item.getGenericReturnType()));
		list.add(new ObjectLevelInspector("default value", item.getDefaultValue()));

		// FIXME: do [][] getParameterAnnotations
		Helper.expandParameterAnnotations(
				item, 
				list, 
				item.getParameterTypes(), 
				item.getParameterAnnotations());
		
		list.add(new SimplePropertyInspector("toGenericString", item.toGenericString()));
		list.add(new SimplePropertyInspector("toString", item.toString()));
	}
}
