/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.inspectors;

import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.SwingUtilities;

import de.admadic.papers.reflection.log.Logger;
import de.admadic.papers.reflection.ui.InspectTreeModel;
import de.admadic.papers.reflection.ui.InspectTreeNode;

/**
 * @author Rainer Schwarze
 */
final public class Util {
	static Logger logger = Logger.getLogger(Util.class);

	private Util() { /* no instance */ }
	
	/**
	 * Returns a String which is built based on the bits being set in the 
	 * parameter mod.
	 * 
	 * @param mod	The modifiers bitfield which shall be converted to a String.
	 * @return	Returns a String representing the modifiers.
	 */
	static public String modifiers2String(int mod) {
		StringBuilder tmp = new StringBuilder();
		if ((mod & Modifier.ABSTRACT)!=0) tmp.append("abstract ");
		if ((mod & Modifier.FINAL)!=0) tmp.append("final ");
		if ((mod & Modifier.INTERFACE)!=0) tmp.append("interface ");
		if ((mod & Modifier.NATIVE)!=0) tmp.append("native ");
		if ((mod & Modifier.PRIVATE)!=0) tmp.append("private ");
		if ((mod & Modifier.PROTECTED)!=0) tmp.append("protected ");
		if ((mod & Modifier.PUBLIC)!=0) tmp.append("public ");
		if ((mod & Modifier.STATIC)!=0) tmp.append("static ");
		if ((mod & Modifier.STRICT)!=0) tmp.append("strict ");
		if ((mod & Modifier.SYNCHRONIZED)!=0) tmp.append("synchronized ");
		if ((mod & Modifier.TRANSIENT)!=0) tmp.append("transient ");
		if ((mod & Modifier.VOLATILE)!=0) tmp.append("volatile ");
		return tmp.toString();
	}

	/**
	 * Returns a String representation of the given boolean.
	 * Actually the function returns "true" or "false" depending on the flag.
	 * 
	 * @param flag
	 * @return	Returns a String representation of the flag.
	 */
	static public String boolean2String(boolean flag) {
		return flag ? "true" : "false";
	}

	/**
	 * Turns an array of T into a List of T.
	 * Actually an ArrayList<T> is created and all elements
	 * of the T[] array are added to it.
	 * 
	 * @param <T>	The type of the array elements.
	 * @param array	The array T[] to be converted to a List<T>.
	 * @return	Returns a List<T> containing all elements of the given array.
	 */
	static public <T> List<T> makeList(T [] array) {
		List<T> list = new ArrayList<T>(array.length);
		Collections.addAll(list, array);
		return list;
	}

	/**
	 * Expands the given tree node.
	 * This method carries out the actual expansion in a background thread.
	 * The results are linked into the node and model inside the EDT.
	 * 
	 * Passing <code>true</code> to <code>stripNullItems</code> lets the expansion
	 * loop check for items in the inspector which are <code>null</code>. 
	 * Such inspectors will then not be included in the expanded list.
	 * 
	 * This method first asks the inspector of the given node whether it is 
	 * already expanded or not. If it is expanded, this method returns immediately.
	 * 
	 * Before doing the expansion in a separate thread, a marker node is added.
	 * The tree node can thus be expanded in the tree view immediately and 
	 * the user is seeing an information, that the expansion is being processed.
	 * Upon completion of the expansion, the "busy marker" node is removed.
	 * 
	 * @param treeNode	The InspectTreeNode whose inspector shall be expanded.
	 * @param treeModel The InspectTreeModel which contains the nodes.
	 * @param stripNullItems Set to true to strip "child inspectors" whose item is <code>null</code>.
	 */
	static public void expand(InspectTreeNode treeNode, InspectTreeModel treeModel, boolean stripNullItems) {
		if (treeNode.getInspector().isExpanded()) return;
		logger.debug("expanding a treeNode (in background)");
		treeModel.insertNodeInto(new InspectTreeNode(new BusyInspector()), treeNode, 0);

		Thread thread = new Thread(
				new Expander(treeNode, treeModel, stripNullItems), 
				"node-expansion-thread");
		thread.setDaemon(true);
		thread.start();
	}

	/**
	 * Expands the inspector of the given tree node. For more information, please
	 * see {@link #expand(InspectTreeNode, InspectTreeModel, boolean)}.
	 * This method calls {@link #expand(InspectTreeNode, InspectTreeModel, boolean)} 
	 * with <code>stripNullItems</code> set to <code>false</code>.
	 * 
	 * @param treeNode
	 * @param treeModel
	 */
	static public void expand(InspectTreeNode treeNode, InspectTreeModel treeModel) {
		expand(treeNode, treeModel, false);
	}

	/**
	 * Provides a class which carries out the expansion of a given node.
	 * The class implements the Runnable interface. An instance of this 
	 * class should be wrapped in a Thread to let the expansion occur in a 
	 * background thread without blocking the EDT. However, you can also 
	 * create an instance and call the run method if blocking is not an issue.
	 * 
	 * @author Rainer Schwarze
	 */
	static class Expander implements Runnable {
		InspectTreeNode treeNode;
		InspectTreeModel treeModel;
		IInspector<?> inspector = null;
		boolean stripNullItems;

		/**
		 * Creates an instance of the expander with the given parameters.
		 * 
		 * @param treeNode	The InspectTreeNode whose inspector shall be expanded.
		 * @param treeModel The InspectTreeModel which contains the nodes.
		 * @param stripNullItems Set to true, to strip the inspectors whose item is null.
		 */
		public Expander(InspectTreeNode treeNode, InspectTreeModel treeModel, boolean stripNullItems) {
			this.treeNode = treeNode;
			this.treeModel = treeModel;
			this.inspector = this.treeNode.getInspector();
			this.stripNullItems = stripNullItems;
		}

		/**
		 * Actually carries out the expansion.
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			// if for some reason there is no inspector, return
			if (inspector==null) return;

			logger.debug("expand thread run() begin");
			// we need a list of the new child inspectors and one of
			// the new child nodes:
			ArrayList<IInspector<?>> list = new ArrayList<IInspector<?>>();
			ArrayList<InspectTreeNode> nodeList = new ArrayList<InspectTreeNode>();

			logger.debug("calling doExpand()...");
			inspector.expand(list);		// let the parent inspector fill the list
			logger.debug("expanded inspectors: " + list.size());

			// now wrap each inspector in an instance of InspectTreeNode:
			for (IInspector<?> i : list) {
				if (stripNullItems) {	// if stripping is allowed, check for null-items:
					if (hasNullItem(i)) continue;
				}
				InspectTreeNode inode = new InspectTreeNode(i);
				nodeList.add(inode);
			}

			// enqueue the insertion of the child nodes to the tree within the EDT:
			final ArrayList<InspectTreeNode> nodeList0 = nodeList;
			logger.debug("enqueuing tree model update");
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					logger.debug("expand thread model update run() begin");
					treeModel.removeAllChildren(treeNode);
					int index = 0;
					for (InspectTreeNode node : nodeList0) {
						treeModel.insertNodeInto(node, treeNode, index++);
					}
					inspector.setExpanded(true);

					for (InspectTreeNode node : nodeList0) {
						if (node.getInspector() instanceof GroupInspector) {
							logger.debug("GroupdInspector detected. enqueueing expansion...");
							expand(node, treeModel);	// enqueue expansion
							logger.debug("enqueueing done...");
						}
					}
					logger.debug("expand thread model update run() end");
				}
			});
			logger.debug("expand thread run() end");
		}

		/**
		 * Checks whether the given inspector has an item which is null.
		 * 
		 * @param i	The inspector to check for a null item.
		 * @return	Returns true, if the inspector supports an item and this is null.
		 */
		private boolean hasNullItem(IInspector<?> i) {
			if (i instanceof AbstractInspector<?>) {
				AbstractInspector<?> dai = (AbstractInspector<?>) i;
				if (dai.getItem()==null) return true;
			}
			return false;
		}
	}


	/**
	 * @param simpleName
	 * @param parameterTypes
	 * @return	Returns a function string (name + parameters).
	 */
	public static String makeFuncString(String simpleName, Class<?>[] parameterTypes) {
		StringBuffer shortsb = new StringBuffer();
		shortsb.append(simpleName);
		shortsb.append("(");
		for (int i=0; i<parameterTypes.length; i++) {
			if (i!=0) shortsb.append(", ");
			shortsb.append(parameterTypes[i].getSimpleName());
		}
		shortsb.append(")");
		return shortsb.toString();
	}
}
