/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.log;

import java.lang.reflect.Method;
import java.security.AccessControlException;

/**
 * Provides a project level encapsulation of logging facilities.
 * 
 * The actual logging handler is set by defining 
 * <code>de.admadic.papers.reflection.log.loggerClass</code> as an option to the VM. If nothing
 * has been specified or the given logger causes errors during initialization,
 * the logger is set to an instance of <code>de.admadic.papers.reflection.log.NullLogger</code>.
 * 
 * To choose the log4j logging, specify the following option for the VM:
 * <code>-Dde.admadic.papers.reflect.log.loggerClass=de.admadic.papers.reflection.log.Log4jLogger</code> .
 * 
 * @author Rainer Schwarze
 */
public class Logger implements ILogger {
	// fields for identifying the "global" Logger class:
	private static String loggerClassName = null;
	private static Class<?> loggerClass = null;
	private static final String LOGGER_PROPNAME = "de.admadic.papers.reflection.log.loggerClass"; //$NON-NLS-1$
	private static final String NULL_LOGGER_CLASSNAME = "de.admadic.papers.reflection.log.NullLogger"; //$NON-NLS-1$

	// instance fields:
	private Class<?> loggedClass = null;
	private ILogger loggerDelegate = null;

	
	/**
	 * Initializes the Logger facilities.
	 */
	public static void initialize() {
		try {
			loggerClassName = System.getProperty(LOGGER_PROPNAME, NULL_LOGGER_CLASSNAME);
			loggerClass = Class.forName(loggerClassName);
		} catch (AccessControlException e) {
			loggerClass = NullLogger.class;
		} catch (ClassNotFoundException e) {
			loggerClass = NullLogger.class;
		}

		try {
			Method m = loggerClass.getMethod("initialize", new Class[]{}); //$NON-NLS-1$
			m.invoke(null, new Object[]{});
		} catch (Throwable e) {
			// no initialization - we fall back to nulllogger
			loggerClassName = NULL_LOGGER_CLASSNAME;
			loggerClass = NullLogger.class;
		}
	}

	/**
	 * Returns a Logger instance for the specified class.
	 * 
	 * @param loggedClass
	 * @return	The Logger instance created for the given class.
	 */
	public static Logger getLogger(Class loggedClass) {
		return new Logger(loggedClass);
	}

	/**
	 * Constructs an instance and creates a delegate logger instance on base 
	 * of the registered logger class.
	 * 
	 * @param loggedClass
	 */
	protected Logger(Class<?> loggedClass) {
		super();
		this.loggedClass = loggedClass;
		try {
			Method m;
			m = loggerClass.getMethod("getLogger", new Class[]{Class.class}); //$NON-NLS-1$
			// this.loggerDelegate = org.apache.log4j.Logger.getLogger(this.loggerClass);
			this.loggerDelegate = (ILogger) m.invoke(null, new Object[]{this.loggedClass});
		} catch (Throwable e) {
			// in case of any error, revert to the NullLogger
			this.loggerDelegate = NullLogger.getLogger(loggedClass);
		}
		if (this.loggerDelegate==null) {
			// if for any reason, no instance has been set, use the NullLogger
			this.loggerDelegate = NullLogger.getLogger(loggedClass);
		}
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#debug(java.lang.Object, java.lang.Throwable)
	 */
	public void debug(Object message, Throwable t) {
		loggerDelegate.debug(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#debug(java.lang.Object)
	 */
	public void debug(Object message) {
		loggerDelegate.debug(message);
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#error(java.lang.Object, java.lang.Throwable)
	 */
	public void error(Object message, Throwable t) {
		loggerDelegate.error(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#error(java.lang.Object)
	 */
	public void error(Object message) {
		loggerDelegate.error(message);
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#fatal(java.lang.Object, java.lang.Throwable)
	 */
	public void fatal(Object message, Throwable t) {
		loggerDelegate.fatal(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#fatal(java.lang.Object)
	 */
	public void fatal(Object message) {
		loggerDelegate.fatal(message);
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#info(java.lang.Object, java.lang.Throwable)
	 */
	public void info(Object message, Throwable t) {
		loggerDelegate.info(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#info(java.lang.Object)
	 */
	public void info(Object message) {
		loggerDelegate.info(message);
	}

	/**
	 * @return	Returns true, if debug level is enabled.
	 * @see de.admadic.papers.reflection.log.ILogger#isDebugEnabled()
	 */
	public boolean isDebugEnabled() {
		return loggerDelegate.isDebugEnabled();
	}

	/**
	 * @return	Returns true, if info level is enabled.
	 * @see de.admadic.papers.reflection.log.ILogger#isInfoEnabled()
	 */
	public boolean isInfoEnabled() {
		return loggerDelegate.isInfoEnabled();
	}

	/**
	 * @return	Returns true, if the trace level is enabled.
	 * @see de.admadic.papers.reflection.log.ILogger#isTraceEnabled()
	 */
	public boolean isTraceEnabled() {
		return loggerDelegate.isTraceEnabled();
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#trace(java.lang.Object, java.lang.Throwable)
	 */
	public void trace(Object message, Throwable t) {
		loggerDelegate.trace(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#trace(java.lang.Object)
	 */
	public void trace(Object message) {
		loggerDelegate.trace(message);
	}

	/**
	 * @param message
	 * @param t
	 * @see de.admadic.papers.reflection.log.ILogger#warn(java.lang.Object, java.lang.Throwable)
	 */
	public void warn(Object message, Throwable t) {
		loggerDelegate.warn(message, t);
	}

	/**
	 * @param message
	 * @see de.admadic.papers.reflection.log.ILogger#warn(java.lang.Object)
	 */
	public void warn(Object message) {
		loggerDelegate.warn(message);
	}
}
