/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;

import de.admadic.papers.reflection.inspectors.BusyInspector;
import de.admadic.papers.reflection.inspectors.ErrorInspector;
import de.admadic.papers.reflection.inspectors.IInspector;
import de.admadic.papers.reflection.inspectors.RootItemInspector;
import de.admadic.papers.reflection.inspectors.RootNodeInspector;
import de.admadic.papers.reflection.inspectors.Util;
import de.admadic.papers.reflection.log.Logger;

/**
 * Provides a panel for inspecting a Class via reflection API.
 * The panel contains a JTree for browsing through the class 
 * member and property structure. The structure tree is not expanded
 * immediately, but step wise for each node which is notified to expand.
 * 
 * (Note that a simple and automated expansion of the full structure tree will
 * cause infinite loops (for instance by running cyclic into java.lang.Class)
 * and thus cause the application to go out of memory. So don't expand the 
 * structure tree automated unless you prevent running into cycles.)
 * 
 * @author Rainer Schwarze
 */
public class InspectPanel extends JPanel implements TreeWillExpandListener, TreeSelectionListener {
	/** */
	private static final long serialVersionUID = 1L;
	static Logger logger = Logger.getLogger(InspectPanel.class);

	String className;	// passed in via constructor

	/*
	 *    _________________________
	 *   [____(path)_______________]
	 * 
	 *   +-------------------------+
	 *   |    (tree)               |^
	 *   :                         :
	 *   |                         |v
	 *   +-------------------------+
	 *   ==========(split)==========
	 *   +-------------------------+^
	 *   |    (details)            |
	 *   +-------------------------+v
	 */
	// the controls as layed out above:
	JLabel pathInfo;
	JTree tree;
	private JScrollPane scrollInspector;
	private JTextArea textDetails;
	private JScrollPane scrollDetails;
	private JSplitPane splitter;

	InspectTreeModel model;

	/**
	 * Creates an instance of the InspectPanel - sets up contents too.
	 * The <code>className<code> parameter must be "fully qualified" - it is 
	 * used as a parameter for <code>Class.forName()</code>.
	 * 
	 * @param className	A String specifying the name of the class to be inspected. 
	 */
	public InspectPanel(String className) {
		super();
		this.className = className;
		initContent();
		initInspectorTree();
	}

	/**
	 * Helper method: create the components inside this panel.
	 */
	void initContent() {
		setLayout(new BorderLayout(10, 10));	// have some gaps

		// build the node path component:
		pathInfo = new JLabel("<node-path>");
		add(pathInfo, BorderLayout.PAGE_START);

		// build the tree component:
		model = new InspectTreeModel(
				new InspectTreeNode(
						new RootNodeInspector()));
		tree = new JTree(model);
		scrollInspector = new JScrollPane(
				tree,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true);
		tree.addTreeWillExpandListener(this);
		tree.addTreeSelectionListener(this);
		tree.setCellRenderer(new InspectNodeTreeCellRenderer());

		// build the details display:
		textDetails = new JTextArea("<details>");
		scrollDetails = new JScrollPane(
				textDetails,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollDetails.setPreferredSize(new Dimension(100, 80));

		// we want tree and details in a splitter:
		splitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false, scrollInspector, scrollDetails);
		splitter.setDividerLocation(300);
		add(splitter, BorderLayout.CENTER);
	}

	/**
	 * Initializes the tree with the top level inspectors for the 
	 * class which shall be inspected by this InpectPanel.
	 */
	protected void initInspectorTree() {
		final InspectTreeNode root = (InspectTreeNode) model.getRoot();
		model.removeAllChildren(root);	// be sure there are no child nodes
		// show that we are busy:
		model.insertNodeInto(
				new InspectTreeNode(new BusyInspector()), 
				root);

		Thread thread = new Thread(new Runnable() {
			public void run() {
				logger.debug("fillroot");
				fillInspectorRootNode(root);
			}
		});
		thread.setDaemon(true);	// don't keep Swing open
		thread.start();
	}

	/**
	 * Helper function: Build the root level elements for the inspector
	 * view. 
	 * Note: This is running outside of the Event Dispatch Thread! Don't 
	 * call Swing methods unless you know what you are doing!
	 * 
	 * @param root	The root node which shall be filled.
	 */
	protected void fillInspectorRootNode(final InspectTreeNode root) {
		Class<?> cls = null;
		IInspector<?> ris = null;
		try {
			cls = Class.forName(className);
			ris = new RootItemInspector(cls);
		} catch (ClassNotFoundException e) {
			ris = new ErrorInspector("Error", e);
		}
		final IInspector<?> ci = ris;

		// update the tree model inside the EDT!
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				model.removeAllChildren(root);
				InspectTreeNode node = model.insertNodeInto(ci, root);
				// be sure to expand the path, or you won't see the nodes:
				tree.expandPath(new TreePath(node.getPath()));
			}
		});
	}

	// ///////////////////////////////////////////
	// various listeners
	
	/**
	 * @param event
	 * @throws ExpandVetoException
	 * @see javax.swing.event.TreeWillExpandListener#treeWillCollapse(javax.swing.event.TreeExpansionEvent)
	 */
	public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
		logger.debug("will collapse");
	}

	/**
	 * @param event
	 * @throws ExpandVetoException
	 * @see javax.swing.event.TreeWillExpandListener#treeWillExpand(javax.swing.event.TreeExpansionEvent)
	 */
	public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
		logger.debug("will expand");
		TreePath path = event.getPath();
		InspectTreeNode node = (InspectTreeNode) path.getLastPathComponent();
		if (node.getInspector().isExpandable()) {
			Util.expand(node, model);
		}
	}

	/**
	 * @param e
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {
		TreePath tp = e.getNewLeadSelectionPath();
		if (tp==null) return;

		// show details for selected node:
		InspectTreeNode tn = (InspectTreeNode) tp.getLastPathComponent();
		IInspector<?> ii = (IInspector<?>) tn.getUserObject();
		textDetails.setText(ii.getDetailedInfo());

		// show path for selected node:
		Object [] path = tp.getPath();
		StringBuilder sb = new StringBuilder();
		for (int i=1; i<path.length; i++) {
			Object onode = path[i];
			InspectTreeNode node = (InspectTreeNode) onode;
			if (sb.length()>0) sb.append(" : ");
			sb.append(node.getInspector().getLabel());
		}
		pathInfo.setText(sb.toString());
	}
}
