/**
 * Copyright (c) 2007 admaDIC GbR. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above 
 *       copyright notice, this list of conditions and the following 
 *       disclaimer in the documentation and/or other materials provided 
 *       with the distribution.
 * 
 *    3. All advertising materials mentioning features or use of this 
 *       software must display the following acknowledgement: 
 * 
 *       This product includes software developed by Rainer Schwarze, admaDIC.
 * 
 *    4. Neither the name Rainer Schwarze, admaDIC nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 */
package de.admadic.papers.reflection.ui;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import de.admadic.papers.reflection.inspectors.IInspector;
import de.admadic.papers.reflection.log.Logger;

/**
 * This class provides a TreeModel for a JTree.
 * 
 * Note that if you change the kind of display (for instance to a TreeTable)
 * you should change this class and {@link InspectTreeNode}. This should
 * limit the number of changes to the rest of the project a bit.
 * 
 * @author Rainer Schwarze
 */
public class InspectTreeModel extends DefaultTreeModel {
	static Logger logger = Logger.getLogger(InspectTreeModel.class);
	/** */
	private static final long serialVersionUID = 1L;

	/**
	 * Creates an instance of the model with the given root node.
	 * 
	 * @param root	The root node in the tree model.
	 */
	public InspectTreeModel(InspectTreeNode root) {
		super(root);
	}

	/**
	 * Convenience method for adding a node and notifying listeners to the 
	 * tree model of the change. The given IInspector is wrapped in an
	 * InspectTreeNode. This method appends the new child node to the
	 * end of the list of children of the parent node.
	 * 
	 * @param newChild	The new child node.
	 * @param parent	The parent node.
	 * @return Returns the node instance which was wrapped around newChild.
	 */
	public InspectTreeNode insertNodeInto(
			IInspector<?> newChild, 
			DefaultMutableTreeNode parent) {
		InspectTreeNode node = new InspectTreeNode(newChild);
		insertNodeInto(node, parent, parent.getChildCount());
		return node;
	}

	/**
	 * Convenience method for adding a node and notifying listeners to the 
	 * tree model of the change. This method appends the new child node to the
	 * end of the list of children of the parent node.
	 * 
	 * @param newChild	The new child node.
	 * @param parent	The parent node.
	 */
	public void insertNodeInto(
			DefaultMutableTreeNode newChild, 
			DefaultMutableTreeNode parent) {
		insertNodeInto(newChild, parent, parent.getChildCount());
	}

	/**
	 * Convenience method for adding a node and notifying listeners to the 
	 * tree model of the change. The <code>index</code> parameter is
	 * a dummy. (It was used for testing various tree table implementations.)
	 * 
	 * @param newChild	The new child node.
	 * @param parent	The parent node.
	 * @param index		Dummy parameter.
	 */
	public void insertNodeInto(DefaultMutableTreeNode newChild, DefaultMutableTreeNode parent, int index) {
		int count = parent.getChildCount();
		parent.add(newChild);
		fireTreeNodesInserted(this, parent.getPath(), new int[]{count}, new Object[]{newChild});
		logger.debug("insertNode : parent=" + parent + " newChild=" + newChild);
	}


	/**
	 * Removes all children from the given node and notifies all listeners
	 * to the tree model about the change.
	 * 
	 * @param node	The InspectTreeNode from which to remove all children.
	 */
	public void removeAllChildren(InspectTreeNode node) {
		int count = node.getChildCount();
		if (count==0) return;
		node.removeAllChildren();
		fireTreeStructureChanged(this, node.getPath(), null, null);
	}

	/**
	 * @param node
	 */
	public void removeNodeFromParent(InspectTreeNode node) {
		InspectTreeNode parent = (InspectTreeNode) node.getParent();
		int index = parent.getIndex(node);
		node.removeFromParent();
		fireTreeStructureChanged(this, parent.getPath(), new int[]{index}, new Object[]{node});
	}

	/**
	 * This method returns whether a given node is a leaf or not by 
	 * calling <code>isLeaf</code> of the node. (The node
	 * is cast to <code>TreeNode</code> for that.)
	 * 
	 * Note that using asksAllowsChildren in the constructor of this class
	 * does not give reliable results in the JTree. I did not analyse this 
	 * situation yet -- Rainer Schwarze, 23.07.2007 
	 * 
	 * @param node	The node for which to check its leaf status.	
	 * @return	Returns whether the given node is a leaf or not.
	 * @see javax.swing.tree.DefaultTreeModel#isLeaf(java.lang.Object)
	 */
	@Override
	public boolean isLeaf(Object node) {
		TreeNode n = (TreeNode) node;
		return n.isLeaf();
	}
}
